package com.zhili.zkservicecenter.configuration;


import com.alibaba.fastjson.JSON;
import com.zhili.zkservicecenter.component.ServiceCenter;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.curator.framework.CuratorFramework;
import org.apache.curator.framework.CuratorFrameworkFactory;
import org.apache.curator.framework.recipes.cache.TreeCache;
import org.apache.curator.framework.recipes.cache.TreeCacheEvent;
import org.apache.curator.framework.recipes.cache.TreeCacheListener;
import org.apache.curator.framework.state.ConnectionState;
import org.apache.curator.framework.state.ConnectionStateListener;
import org.apache.curator.retry.RetryForever;
import org.apache.zookeeper.CreateMode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.nio.charset.StandardCharsets;

/**
 * @author ：HuangBin
 * @description：TODO
 * @date ：2023/1/3 17:27
 */
@Configuration
@Slf4j
public class ZookeeperConfig {
    @Value("${zookeeper.server}")
    String zkAddress;

    @Autowired
    ServerConfig serverConfig;

    @Bean
    public CuratorFramework curatorFramework() throws Exception {
        log.info("serverConfig:{}", serverConfig);
        CuratorFramework client = CuratorFrameworkFactory.builder()
                .connectString(zkAddress)
                .sessionTimeoutMs(5000)
                .connectionTimeoutMs(5000)
                .retryPolicy(new RetryForever(5000)).build();
        //重连的时候都创建已有的临时节点。
        client.getConnectionStateListenable().addListener(new ConnectionStateListener() {
            @SneakyThrows
            @Override
            public void stateChanged(CuratorFramework cli, ConnectionState state) {
                switch (state) {
                    case RECONNECTED:
                        log.info("zk RECONNECTED");
                        String instanceNode = serverConfig.getZkRoot() + "/" + serverConfig.getServiceName() + "/" + serverConfig.getInstanceId();
                        cli.create().creatingParentsIfNeeded()
                                .withMode(CreateMode.EPHEMERAL)
                                .forPath(instanceNode, JSON.toJSONString(serverConfig).getBytes(StandardCharsets.UTF_8));
                        break;
                    default:
                        log.info("zk conn other state:" + state.name());
                        break;
                }
            }
        });
        client.start();
        client.blockUntilConnected();
        log.info("zk connected");

        //创建自身业务中心服务器节点
        String createInstancePath = serverConfig.getZkRoot() + "/" + serverConfig.getServiceName() + "/" + serverConfig.getServiceName() + "-";
        String instanceNode = client.create()
                .creatingParentsIfNeeded()
                .withMode(CreateMode.EPHEMERAL_SEQUENTIAL)
                .forPath(createInstancePath, JSON.toJSONString(serverConfig).getBytes(StandardCharsets.UTF_8));
        log.info("instanceNode:" + instanceNode);
        String instanceId = instanceNode.substring(instanceNode.lastIndexOf("/") + 1);
        serverConfig.setInstanceId(instanceId);

        // 监听所有服务器目录，包括自身
        String watchPath = serverConfig.getZkRoot();
        TreeCache serverNodesCache = new TreeCache(client, watchPath);
        TreeCacheListener serverNodesListener = new TreeCacheListener() {
            @Override
            public void childEvent(CuratorFramework client, TreeCacheEvent event) throws Exception {
                String[] split;
                String serverRole;
                String serverId;
                log.info("收到zookeeper事件：" + event.getType().name() + ";" + event);
                switch (event.getType()) {
                    case NODE_ADDED:
                        split = event.getData().getPath().split("/");
                        if(split.length<4){
                            break;
                        }
                        serverRole = split[split.length - 2];
                        serverId = split[split.length - 1];
                        ServerConfig data = JSON.parseObject(new String(event.getData().getData()), ServerConfig.class);
                        log.info("服务器上线了({})：{},data:{}", serverRole, serverId, data);
                        ServiceCenter.addServer(serverRole, serverId, data);
                        break;
                    case NODE_REMOVED:
                        split = event.getData().getPath().split("/");
                        if(split.length<4){
                            break;
                        }
                        serverRole = split[split.length - 2];
                        serverId = split[split.length - 1];
                        log.info("服务器下线了({})：{}", serverRole, serverId);
                        ServiceCenter.remServer(serverRole, serverId);
                        break;
                    default:
                        break;
                }
            }
        };
        serverNodesCache.getListenable().addListener(serverNodesListener);
        serverNodesCache.start();

        return client;
    }
}
